@@ -1,3 +1,4 @@
+// clang-format off
 /**
  * KDiff3 - Text Diff And Merge Tool
  *
@@ -5,10 +6,11 @@
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
  */
+// clang-format on
 
 #include "DefaultFileAccessJobHandler.h"
 
-#include "common.h"
+#include "compat.h"
 #include "defmac.h"
 #include "fileaccess.h"
 #include "IgnoreList.h"
@@ -25,40 +27,48 @@
 #include <QDir>
 #include <QUrl>
 
-#include <KJob>
 #include <KIO/CopyJob>
-#include <KIO/Job>
+#include <KIO/FileCopyJob>
+#include <KIO/ListJob>
+#include <KIO/MkdirJob>
+#include <KIO/SimpleJob>
+#include <KIO/StatJob>
+#include <KIO/TransferJob>
+#include <KJob>
 #include <KLocalizedString>
 
-#include <KMessageBox>
 #include <KIO/JobUiDelegate>
+#include <KMessageBox>
 
 bool DefaultFileAccessJobHandler::stat(bool bWantToWrite)
 {
     m_bSuccess = false;
+#if HAS_KFKIO
     mFileAccess->setStatusText(QString());
-#if KF_VERSION < KF_VERSION_CHECK(5, 69, 0)
-    KIO::StatJob* pStatJob = KIO::stat(mFileAccess->url(),
-                                       bWantToWrite ? KIO::StatJob::DestinationSide : KIO::StatJob::SourceSide,
-                                       2/*all details*/, KIO::HideProgressInfo);
-#else
+
+    //For testing kf6 build.
+    #if KF_VERSION < KF_VERSION_CHECK(5,240,0)
     KIO::StatJob* pStatJob = KIO::statDetails(mFileAccess->url(),
                                        bWantToWrite ? KIO::StatJob::DestinationSide : KIO::StatJob::SourceSide,
                                        KIO::StatDefaultDetails, KIO::HideProgressInfo);
+    #else
+    KIO::StatJob* pStatJob = KIO::stat(mFileAccess->url(),
+                                       bWantToWrite ? KIO::StatJob::DestinationSide : KIO::StatJob::SourceSide,
+                                       KIO::StatDefaultDetails, KIO::HideProgressInfo);
 
-#endif
-
+    #endif
     chk_connect(pStatJob, &KIO::StatJob::result, this, &DefaultFileAccessJobHandler::slotStatResult);
     chk_connect(pStatJob, &KIO::StatJob::finished, this, &DefaultFileAccessJobHandler::slotJobEnded);
 
     ProgressProxy::enterEventLoop(pStatJob, i18n("Getting file status: %1", mFileAccess->prettyAbsPath()));
-
+#endif
     return m_bSuccess;
 }
 
+#if HAS_KFKIO
 void DefaultFileAccessJobHandler::slotStatResult(KJob* pJob)
 {
-    int err = pJob->error();
+    qint32 err = pJob->error();
     if(err != KJob::NoError)
     {
         qCDebug(kdiffFileAccess) << "slotStatResult: pJob->error() = " << pJob->error();
@@ -84,18 +94,20 @@ void DefaultFileAccessJobHandler::slotStatResult(KJob* pJob)
         m_bSuccess = mFileAccess->isValid();
     }
 }
+#endif
 
 bool DefaultFileAccessJobHandler::get(void* pDestBuffer, long maxLength)
 {
     ProgressProxyExtender pp; // Implicitly used in slotPercent()
 
-    if(maxLength > 0 && !pp.wasCancelled())
+    if(maxLength > 0 && !ProgressProxy::wasCancelled())
     {
+        m_bSuccess = false;
+#if HAS_KFKIO
         KIO::TransferJob* pJob = KIO::get(mFileAccess->url(), KIO::NoReload);
         m_transferredBytes = 0;
         m_pTransferBuffer = (char*)pDestBuffer;
         m_maxLength = maxLength;
-        m_bSuccess = false;
         mFileAccess->setStatusText(QString());
 
         chk_connect(pJob, &KIO::TransferJob::result, this, &DefaultFileAccessJobHandler::slotSimpleJobResult);
@@ -103,13 +115,15 @@ bool DefaultFileAccessJobHandler::get(void* pDestBuffer, long maxLength)
         chk_connect(pJob, &KIO::TransferJob::data, this, &DefaultFileAccessJobHandler::slotGetData);
         chk_connect(pJob, SIGNAL(percent(KJob*,ulong)), &pp, SLOT(slotPercent(KJob*,ulong)));
 
-        ProgressProxy::enterEventLoop(pJob, i18nc("Mesage for progress dialog %1 = path to file", "Reading file: %1", mFileAccess->prettyAbsPath()));
+        ProgressProxy::enterEventLoop(pJob, i18nc("Message for progress dialog %1 = path to file", "Reading file: %1", mFileAccess->prettyAbsPath()));
+#endif
         return m_bSuccess;
     }
     else
         return true;
 }
 
+#if HAS_KFKIO
 void DefaultFileAccessJobHandler::slotGetData(KJob* pJob, const QByteArray& newData)
 {
     if(pJob->error() != KJob::NoError)
@@ -124,18 +138,20 @@ void DefaultFileAccessJobHandler::slotGetData(KJob* pJob, const QByteArray& newD
         m_transferredBytes += length;
     }
 }
+#endif
 
-bool DefaultFileAccessJobHandler::put(const void* pSrcBuffer, long maxLength, bool bOverwrite, bool bResume, int permissions)
+bool DefaultFileAccessJobHandler::put(const void* pSrcBuffer, long maxLength, bool bOverwrite, bool bResume, qint32 permissions)
 {
     ProgressProxyExtender pp; // Implicitly used in slotPercent()
     if(maxLength > 0)
     {
+        m_bSuccess = false;
+#if HAS_KFKIO
         KIO::TransferJob* pJob = KIO::put(mFileAccess->url(), permissions,
                                           KIO::HideProgressInfo | (bOverwrite ? KIO::Overwrite : KIO::DefaultFlags) | (bResume ? KIO::Resume : KIO::DefaultFlags));
         m_transferredBytes = 0;
         m_pTransferBuffer = (char*)pSrcBuffer;
         m_maxLength = maxLength;
-        m_bSuccess = false;
         mFileAccess->setStatusText(QString());
 
         chk_connect(pJob, &KIO::TransferJob::result, this, &DefaultFileAccessJobHandler::slotPutJobResult);
@@ -143,13 +159,15 @@ bool DefaultFileAccessJobHandler::put(const void* pSrcBuffer, long maxLength, bo
         chk_connect(pJob, &KIO::TransferJob::dataReq, this, &DefaultFileAccessJobHandler::slotPutData);
         chk_connect(pJob, SIGNAL(percent(KJob*,ulong)), &pp, SLOT(slotPercent(KJob*,ulong)));
 
-        ProgressProxy::enterEventLoop(pJob, i18nc("Mesage for progress dialog %1 = path to file", "Writing file: %1", mFileAccess->prettyAbsPath()));
+        ProgressProxy::enterEventLoop(pJob, i18nc("Message for progress dialog %1 = path to file", "Writing file: %1", mFileAccess->prettyAbsPath()));
+#endif
         return m_bSuccess;
     }
     else
         return true;
 }
 
+#if HAS_KFKIO
 void DefaultFileAccessJobHandler::slotPutData(KIO::Job* pJob, QByteArray& data)
 {
     if(pJob->error() != KJob::NoError)
@@ -166,10 +184,10 @@ void DefaultFileAccessJobHandler::slotPutData(KIO::Job* pJob, QByteArray& data)
         */
         qint64 maxChunkSize = 100000;
         qint64 length = std::min(maxChunkSize, m_maxLength - m_transferredBytes);
-        data.resize((QtSizeType)length);
-        if(data.size() == (QtSizeType)length)
+        if(length > 0)
         {
-            if(length > 0)
+            data.resize((QtSizeType)length);
+            if(data.size() == (QtSizeType)length)
             {
                 ::memcpy(data.data(), m_pTransferBuffer + m_transferredBytes, data.size());
                 m_transferredBytes += length;
@@ -177,7 +195,7 @@ void DefaultFileAccessJobHandler::slotPutData(KIO::Job* pJob, QByteArray& data)
         }
         else
         {
-            KMessageBox::error(g_pProgressDialog.get(), i18n("Out of memory"));
+            KMessageBox::error(g_pProgressDialog, i18n("Out of memory"));
             data.resize(0);
             m_bSuccess = false;
         }
@@ -196,6 +214,7 @@ void DefaultFileAccessJobHandler::slotPutJobResult(KJob* pJob)
         m_bSuccess = (m_transferredBytes == m_maxLength); // Special success condition
     }
 }
+#endif
 
 bool DefaultFileAccessJobHandler::mkDirImp(const QString& dirName)
 {
@@ -210,11 +229,13 @@ bool DefaultFileAccessJobHandler::mkDirImp(const QString& dirName)
     else
     {
         m_bSuccess = false;
+#if HAS_KFKIO
         KIO::SimpleJob* pJob = KIO::mkdir(dir.url());
         chk_connect(pJob, &KIO::SimpleJob::result, this, &DefaultFileAccessJobHandler::slotSimpleJobResult);
         chk_connect(pJob, &KIO::SimpleJob::finished, this, &DefaultFileAccessJobHandler::slotJobEnded);
 
-        ProgressProxy::enterEventLoop(pJob, i18nc("Mesage for progress dialog %1 = path to file", "Making folder: %1", dirName));
+        ProgressProxy::enterEventLoop(pJob, i18nc("Message for progress dialog %1 = path to file", "Making folder: %1", dirName));
+#endif
         return m_bSuccess;
     }
 }
@@ -232,11 +253,13 @@ bool DefaultFileAccessJobHandler::rmDirImp(const QString& dirName)
     else
     {
         m_bSuccess = false;
+#if HAS_KFKIO
         KIO::SimpleJob* pJob = KIO::rmdir(fa.url());
         chk_connect(pJob, &KIO::SimpleJob::result, this, &DefaultFileAccessJobHandler::slotSimpleJobResult);
         chk_connect(pJob, &KIO::SimpleJob::finished, this, &DefaultFileAccessJobHandler::slotJobEnded);
 
-        ProgressProxy::enterEventLoop(pJob, i18nc("Mesage for progress dialog %1 = path to file", "Removing folder: %1", dirName));
+        ProgressProxy::enterEventLoop(pJob, i18nc("Message for progress dialog %1 = path to file", "Removing folder: %1", dirName));
+#endif
         return m_bSuccess;
     }
 }
@@ -248,11 +271,16 @@ bool DefaultFileAccessJobHandler::removeFile(const QUrl& fileName)
     else
     {
         m_bSuccess = false;
+#if HAS_KFKIO
         KIO::SimpleJob* pJob = KIO::file_delete(fileName, KIO::HideProgressInfo);
         chk_connect(pJob, &KIO::SimpleJob::result, this, &DefaultFileAccessJobHandler::slotSimpleJobResult);
         chk_connect(pJob, &KIO::SimpleJob::finished, this, &DefaultFileAccessJobHandler::slotJobEnded);
 
-        ProgressProxy::enterEventLoop(pJob, i18nc("Mesage for progress dialog %1 = path to file", "Removing file: %1", FileAccess::prettyAbsPath(fileName)));
+        ProgressProxy::enterEventLoop(pJob, i18nc("Message for progress dialog %1 = path to file", "Removing file: %1", FileAccess::prettyAbsPath(fileName)));
+#else
+        if(FileAccess::isLocal(fileName))
+            m_bSuccess = QFile::remove(fileName)
+#endif
         return m_bSuccess;
     }
 }
@@ -264,12 +292,18 @@ bool DefaultFileAccessJobHandler::symLink(const QUrl& linkTarget, const QUrl& li
     else
     {
         m_bSuccess = false;
+#if HAS_KFKIO
         KIO::CopyJob* pJob = KIO::link(linkTarget, linkLocation, KIO::HideProgressInfo);
         chk_connect(pJob, &KIO::CopyJob::result, this, &DefaultFileAccessJobHandler::slotSimpleJobResult);
         chk_connect(pJob, &KIO::CopyJob::finished, this, &DefaultFileAccessJobHandler::slotJobEnded);
 
         ProgressProxy::enterEventLoop(pJob,
                                       i18n("Creating symbolic link: %1 -> %2", FileAccess::prettyAbsPath(linkLocation), FileAccess::prettyAbsPath(linkTarget)));
+#else
+        //TODO: Not implemented.
+        //if(FileAccess::isLocal(fileName))
+        //    m_bSuccess = QFile::link(linkTarget, linkLocation)
+#endif
         return m_bSuccess;
     }
 }
@@ -286,7 +320,7 @@ bool DefaultFileAccessJobHandler::rename(const FileAccess& destFile)
     else
     {
         ProgressProxyExtender pp;
-        int permissions = -1;
+        qint32 permissions = -1;
         m_bSuccess = false;
         KIO::FileCopyJob* pJob = KIO::file_move(mFileAccess->url(), destFile.url(), permissions, KIO::HideProgressInfo);
         chk_connect(pJob, &KIO::FileCopyJob::result, this, &DefaultFileAccessJobHandler::slotSimpleJobResult);
@@ -299,10 +333,8 @@ bool DefaultFileAccessJobHandler::rename(const FileAccess& destFile)
     }
 }
 
-void DefaultFileAccessJobHandler::slotJobEnded(KJob* pJob)
+void DefaultFileAccessJobHandler::slotJobEnded([[maybe_unused]] KJob* pJob)
 {
-    Q_UNUSED(pJob);
-
     ProgressProxy::exitEventLoop(); // Close the dialog, return from exec()
 }
 
@@ -329,11 +361,11 @@ bool DefaultFileAccessJobHandler::copyFile(const QString& inDest)
     mFileAccess->setStatusText(QString());
     if(!mFileAccess->isNormal() || !dest.isNormal()) return false;
 
-    int permissions = (mFileAccess->isExecutable() ? 0111 : 0) + (mFileAccess->isWritable() ? 0222 : 0) + (mFileAccess->isReadable() ? 0444 : 0);
+    qint32 permissions = (mFileAccess->isExecutable() ? 0111 : 0) + (mFileAccess->isWritable() ? 0222 : 0) + (mFileAccess->isReadable() ? 0444 : 0);
     m_bSuccess = false;
     KIO::FileCopyJob* pJob = KIO::file_copy(mFileAccess->url(), dest.url(), permissions, KIO::HideProgressInfo|KIO::Overwrite);
     chk_connect(pJob, &KIO::FileCopyJob::result, this, &DefaultFileAccessJobHandler::slotSimpleJobResult);
-    chk_connect(pJob, SIGNAL(percent(KJob*,ulong)), &pp, SLOT(slotPercent(KJob*,ulong)));
+    chk_connect(pJob, &KIO::FileCopyJob::percentChanged, &pp, &ProgressProxyExtender::slotPercent);
     chk_connect(pJob, &KIO::FileCopyJob::finished, this, &DefaultFileAccessJobHandler::slotJobEnded);
 
     ProgressProxy::enterEventLoop(pJob,
@@ -356,10 +388,10 @@ bool DefaultFileAccessJobHandler::listDir(DirectoryList* pDirList, bool bRecursi
     m_filePattern = filePattern;
     m_dirAntiPattern = dirAntiPattern;
 
-    if(pp.wasCancelled())
+    if(ProgressProxy::wasCancelled())
         return true; // Cancelled is not an error.
 
-    pp.setInformation(i18n("Reading folder: %1", mFileAccess->absoluteFilePath()), 0, false);
+    ProgressProxy::setInformation(i18nc("Status message", "Reading folder: %1", mFileAccess->absoluteFilePath()), 0, false);
     qCInfo(kdiffFileAccess) << "Reading folder: " << mFileAccess->absoluteFilePath();
 
     if(mFileAccess->isLocal())
@@ -386,7 +418,7 @@ bool DefaultFileAccessJobHandler::listDir(DirectoryList* pDirList, bool bRecursi
         {
             for(const QFileInfo& fi: fiList) // for each file...
             {
-                if(pp.wasCancelled())
+                if(ProgressProxy::wasCancelled())
                     break;
 
                 assert(fi.fileName() != "." && fi.fileName() != "..");
@@ -401,8 +433,11 @@ bool DefaultFileAccessJobHandler::listDir(DirectoryList* pDirList, bool bRecursi
     else
     {
         KIO::ListJob* pListJob = nullptr;
+#if KF_VERSION < KF_VERSION_CHECK(5, 240, 0)
         pListJob = KIO::listDir(mFileAccess->url(), KIO::HideProgressInfo, true /*bFindHidden*/);
-
+#else
+        pListJob = KIO::listDir(mFileAccess->url(), KIO::HideProgressInfo, KIO::ListJob::ListFlag::IncludeHidden /*bFindHidden*/);
+#endif
         m_bSuccess = false;
         if(pListJob != nullptr)
         {
