@@ -1,3 +1,4 @@
+// clang-format off
 /**
  * KDiff3 - Text Diff And Merge Tool
  *
@@ -5,6 +6,7 @@
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
  */
+// clang-format on
 
 #ifndef DEFAULTFILEACCESSJOBHANDLER_H
 #define DEFAULTFILEACCESSJOBHANDLER_H
@@ -33,7 +35,7 @@ class DefaultFileAccessJobHandler: public FileAccessJobHandler
     FileAccessJobHandler* copy(FileAccess* inFileAccess) override { return new DefaultFileAccessJobHandler(inFileAccess);}
 
     bool get(void* pDestBuffer, long maxLength) override;
-    bool put(const void* pSrcBuffer, long maxLength, bool bOverwrite, bool bResume = false, int permissions = -1) override;
+    bool put(const void* pSrcBuffer, long maxLength, bool bOverwrite, bool bResume = false, qint32 permissions = -1) override;
     bool stat(bool bWantToWrite = false) override;
     bool copyFile(const QString& dest) override;
     bool rename(const FileAccess& dest) override;
