@@ -1,3 +1,4 @@
+// clang-format off
 /*
  * KDiff3 - Text Diff And Merge Tool
  *
@@ -25,9 +26,17 @@ class LineRef
     //cppcheck-suppress noExplicitConstructor
     constexpr inline LineRef(const LineType i) { mLineNumber = i; }
     //cppcheck-suppress noExplicitConstructor
-    inline LineRef(const qint64 i)
+    constexpr inline LineRef(const quint64 i)
     {
-        if(i <= TYPE_MAX(LineType) && i >= 0)
+        if(i <= limits<LineType>::max())
+            mLineNumber = (LineType)i;
+        else
+            mLineNumber = invalid;
+    }
+    //cppcheck-suppress noExplicitConstructor
+    constexpr inline LineRef(const qint64 i)
+    {
+        if(i <= limits<LineType>::max() && i >= 0)
             mLineNumber = (LineType)i;
         else
             mLineNumber = invalid;
@@ -35,6 +44,8 @@ class LineRef
 
     inline operator LineType() const noexcept { return mLineNumber; }
 
+    inline operator SafeInt<LineType>() const noexcept { return mLineNumber; }
+
     inline LineRef& operator=(const LineType lineIn) noexcept
     {
         mLineNumber = lineIn;
@@ -53,7 +64,7 @@ class LineRef
         return *this;
     };
 
-    const LineRef operator++(qint32) noexcept
+    const LineRef operator++(int) noexcept
     {
         LineRef line(*this);
         ++mLineNumber;
@@ -66,7 +77,7 @@ class LineRef
         return *this;
     };
 
-    const LineRef operator--(qint32) noexcept
+    const LineRef operator--(int) noexcept
     {
         LineRef line(*this);
         --mLineNumber;
@@ -76,25 +87,24 @@ class LineRef
     [[nodiscard]] inline bool isValid() const noexcept { return mLineNumber != invalid; }
 
   private:
-    SafeInt<LineType> mLineNumber = invalid;
+    SafeSignedRange<LineType, invalid> mLineNumber = invalid;
 };
 
 /*
     This is here because its easy to unknowingly break these conditions. The resulting
-    compiler errors are numerous and may be a bit cryptic if you aren't famialar with the C++ language.
+    compiler errors are numerous and may be a bit cryptic if you aren't familiar with the C++ language.
     Also some IDEs with clangd or ccls integration can automatically check static_asserts
     without doing a full compile.
 */
-static_assert(std::is_copy_constructible<LineRef>::value, "LineRef must be copy constuctible.");
+static_assert(std::is_copy_constructible<LineRef>::value, "LineRef must be copy constructible.");
 static_assert(std::is_copy_assignable<LineRef>::value, "LineRef must copy assignable.");
 static_assert(std::is_move_constructible<LineRef>::value, "LineRef must be move constructible.");
 static_assert(std::is_move_assignable<LineRef>::value, "LineRef must be move assignable.");
-static_assert(std::is_convertible<LineRef, qint32>::value, "Can not convert LineRef to qint32.");
-static_assert(std::is_convertible<qint32, LineRef>::value, "Can not convert qint32 to LineRef.");
+static_assert(std::is_convertible<LineRef, int>::value, "Can not convert LineRef to int.");
+static_assert(std::is_convertible<int, LineRef>::value, "Can not convert int to LineRef.");
 
-typedef LineRef::LineType LineCount;
-typedef LineRef::LineType LineIndex;
+using LineType = LineRef::LineType;
 
-//Break in an obvious way if way cann't get LineCounts from Qt supplied ints without overflow issues.
-static_assert(sizeof(LineCount) >= sizeof(qint32)); //Generally assumed by KDiff3
+//Break in an obvious way if way can't get LineCounts from Qt supplied ints without overflow issues.
+static_assert(sizeof(LineType) >= sizeof(qint32)); //Generally assumed by KDiff3
 #endif
