@@ -20,6 +20,7 @@
 #include <algorithm>           // for min
 #include <cstdlib>
 #include <ctype.h>
+#include <exception>
 #include <memory>
 #include <optional>
 #include <utility>             // for swap
@@ -33,6 +34,8 @@
 #include <QSharedPointer>
 #include <QTextStream>
 
+extern std::unique_ptr<Options> gOptions;
+
 constexpr bool g_bIgnoreWhiteSpace = true;
 
 QSharedPointer<DiffBufferInfo> Diff3Line::m_pDiffBufferInfo = QSharedPointer<DiffBufferInfo>::create();
@@ -135,17 +138,12 @@ void Diff3LineList::calcDiff3LineListUsingAB(const DiffList* pDiffListAB)
 {
     // First make d3ll for AB (from pDiffListAB)
 
-    DiffList::const_iterator i = pDiffListAB->begin();
     LineRef lineA = 0;
     LineRef lineB = 0;
-    Diff d;
 
     qCInfo(kdiffMain) << "Enter: calcDiff3LineListUsingAB";
-    while(i != pDiffListAB->end())
+    for(Diff d: *pDiffListAB)
     {
-        d = *i;
-        ++i;
-
         while(d.numberOfEquals() > 0)
         {
             Diff3Line d3l;
@@ -209,18 +207,13 @@ void Diff3LineList::calcDiff3LineListUsingAC(const DiffList* pDiffListAC)
     ////////////////
     // Now insert data from C using pDiffListAC
 
-    DiffList::const_iterator i = pDiffListAC->begin();
     Diff3LineList::iterator i3 = begin();
     LineRef lineA = 0;
     LineRef lineC = 0;
-    Diff d;
 
-    while(i != pDiffListAC->end())
+    for(Diff d: *pDiffListAC)
     {
-        d = *i;
-        ++i;
-
-        assert(d.diff1() <= TYPE_MAX(LineRef::LineType) && d.diff2() <= TYPE_MAX(LineRef::LineType));
+        assert(d.diff1() <= limits<LineType>::max() && d.diff2() <= limits<LineType>::max());
 
         while(d.numberOfEquals() > 0)
         {
@@ -254,11 +247,7 @@ void Diff3LineList::calcDiff3LineListUsingAC(const DiffList* pDiffListAC)
             ++lineC;
         }
 
-        while(d.diff1() > 0)
-        {
-            d.adjustDiff1(-1);
-            ++lineA;
-        }
+        lineA += (LineRef)d.diff1();
 
         while(d.diff2() > 0)
         {
@@ -282,18 +271,13 @@ void Diff3LineList::calcDiff3LineListUsingBC(const DiffList* pDiffListBC)
     // If a line from C equals a line from B but not A, this
     // information will be used here.
 
-    DiffList::const_iterator i = pDiffListBC->begin();
     Diff3LineList::iterator i3b = begin();
     Diff3LineList::iterator i3c = begin();
     LineRef lineB = 0;
     LineRef lineC = 0;
-    Diff d;
 
-    while(i == pDiffListBC->end())
+    for(Diff d: *pDiffListBC)
     {
-        d = *i;
-        ++i;
-
         while(d.numberOfEquals() > 0)
         {
             Diff3Line d3l;
@@ -330,7 +314,7 @@ void Diff3LineList::calcDiff3LineListUsingBC(const DiffList* pDiffListBC)
                 if(i3c1 == i3b && !i3b->isEqualAB()) // i3c before i3b
                 {
                     Diff3LineList::iterator i3 = i3c;
-                    qint32 nofDisturbingLines = 0;
+                    quint32 nofDisturbingLines = 0;
                     while(i3 != i3b && i3 != end())
                     {
                         if(i3->getLineB().isValid())
@@ -338,7 +322,7 @@ void Diff3LineList::calcDiff3LineListUsingBC(const DiffList* pDiffListBC)
                         ++i3;
                     }
 
-                    if(nofDisturbingLines > 0) //&& nofDisturbingLines < d.nofEquals*d.nofEquals+4 )
+                    if(nofDisturbingLines > 0)
                     {
                         Diff3LineList::iterator i3_last_equal_A = end();
 
@@ -389,24 +373,23 @@ void Diff3LineList::calcDiff3LineListUsingBC(const DiffList* pDiffListBC)
 
                             ++i3;
                         }
-                        nofDisturbingLines = 0;
+                        //Not currently needed as we get here everytime nofDisturbingLines > 0
+                        //nofDisturbingLines = 0;
                     }
 
-                    if(nofDisturbingLines == 0)
-                    {
-                        // Yes, the line from B can be moved.
-                        i3b->setLineB(LineRef::invalid); // This might leave an empty line: removed later.
-                        i3b->bAEqB = false;
-                        i3b->bBEqC = false;
-                        i3c->setLineB(lineB);
-                        i3c->bBEqC = true;
-                        i3c->bAEqB = i3c->isEqualAC();
-                    }
+                    // Was checking if nofDisturbingLines == 0 which would always be true. See above.
+                    // Yes, the line from B can be moved.
+                    i3b->setLineB(LineRef::invalid); // This might leave an empty line: removed later.
+                    i3b->bAEqB = false;
+                    i3b->bBEqC = false;
+                    i3c->setLineB(lineB);
+                    i3c->bBEqC = true;
+                    i3c->bAEqB = i3c->isEqualAC();
                 }
                 else if(i3b1 == i3c && !i3c->isEqualAC())
                 {
                     Diff3LineList::iterator i3 = i3b;
-                    qint32 nofDisturbingLines = 0;
+                    quint32 nofDisturbingLines = 0;
                     while(i3 != i3c && i3 != end())
                     {
                         if(i3->getLineC().isValid())
@@ -414,7 +397,7 @@ void Diff3LineList::calcDiff3LineListUsingBC(const DiffList* pDiffListBC)
                         ++i3;
                     }
 
-                    if(nofDisturbingLines > 0) //&& nofDisturbingLines < d.nofEquals*d.nofEquals+4 )
+                    if(nofDisturbingLines > 0)
                     {
                         Diff3LineList::iterator i3_last_equal_A = end();
 
@@ -465,19 +448,18 @@ void Diff3LineList::calcDiff3LineListUsingBC(const DiffList* pDiffListBC)
 
                             ++i3;
                         }
-                        nofDisturbingLines = 0;
+                        //Not currently needed as we get here everytime nofDisturbingLines > 0
+                        //nofDisturbingLines = 0;
                     }
 
-                    if(nofDisturbingLines == 0)
-                    {
-                        // Yes, the line from C can be moved.
-                        i3c->setLineC(LineRef::invalid); // This might leave an empty line: removed later.
-                        i3c->bAEqC = false;
-                        i3c->bBEqC = false;
-                        i3b->setLineC(lineC);
-                        i3b->bBEqC = true;
-                        i3b->bAEqC = i3b->isEqualAB();
-                    }
+                    // Only do this if nofDisturbingLines == 0 currently always true
+                    // Yes, the line from C can be moved.
+                    i3c->setLineC(LineRef::invalid); // This might leave an empty line: removed later.
+                    i3c->bAEqC = false;
+                    i3c->bBEqC = false;
+                    i3b->setLineC(lineC);
+                    i3b->bBEqC = true;
+                    i3b->bAEqC = i3b->isEqualAB();
                 }
             }
 
@@ -517,11 +499,7 @@ void Diff3LineList::calcDiff3LineListUsingBC(const DiffList* pDiffListBC)
             }
         }
 
-        while(d.diff2() > 0)
-        {
-            d.adjustDiff2(-1);
-            ++lineC;
-        }
+        lineC += (LineRef)d.diff2();
     }
     /*
    Diff3LineList::iterator it = d3ll.begin();
@@ -571,7 +549,7 @@ void ManualDiffHelpList::insertEntry(e_SrcSelector winIdx, LineRef firstLine, Li
                 l1.invalidate();
                 l2.invalidate();
             }
-            if(firstLine < l1 && lastLine < l1)
+            else if(firstLine < l1 && lastLine < l1)
             {
                 // insert before this position
                 insert(i, mdhe);
@@ -585,25 +563,25 @@ void ManualDiffHelpList::insertEntry(e_SrcSelector winIdx, LineRef firstLine, Li
     }
 
     // Now make the list compact
-    for(e_SrcSelector wIdx = e_SrcSelector::A; wIdx != e_SrcSelector::Invalid; wIdx = nextSelector(wIdx))
+    for(i = begin(); i != end();)
     {
-        ManualDiffHelpList::iterator iEmpty = begin();
-        for(i = begin(); i != end(); ++i)
+        ManualDiffHelpList::iterator next = std::next(i);
+        if(next != end())
         {
-            if(iEmpty->firstLine(wIdx).isValid())
+            for(e_SrcSelector wIdx = e_SrcSelector::A; wIdx != e_SrcSelector::Invalid; wIdx = nextSelector(wIdx))
             {
-                ++iEmpty;
-                continue;
-            }
-            if(i->firstLine(wIdx).isValid()) // Current item is not empty -> move it to the empty place
-            {
-                std::swap(iEmpty->firstLine(wIdx), i->firstLine(wIdx));
-                std::swap(iEmpty->lastLine(wIdx), i->lastLine(wIdx));
-                ++iEmpty;
+                if(!i->firstLine(wIdx).isValid() && next->firstLine(wIdx).isValid())
+                {
+                    std::swap(i->firstLine(wIdx), next->firstLine(wIdx));
+                    std::swap(i->lastLine(wIdx), next->lastLine(wIdx));
+                }
             }
         }
+        //Delete completely empty entries
+        if(*i == ManualDiffHelpEntry())
+            erase(i);
+        i = next;
     }
-    remove(ManualDiffHelpEntry()); // Remove all completely empty items.
 }
 
 bool ManualDiffHelpEntry::isValidMove(LineRef line1, LineRef line2, e_SrcSelector winIdx1, e_SrcSelector winIdx2) const
@@ -641,8 +619,7 @@ qint32 ManualDiffHelpEntry::calcManualDiffFirstDiff3LineIdx(const Diff3LineVecto
     return -1;
 }
 
-void DiffList::runDiff(const std::shared_ptr<LineDataVector> &p1, const size_t index1, LineRef size1, const std::shared_ptr<LineDataVector> &p2, const size_t index2, LineRef size2,
-                    const QSharedPointer<Options> &pOptions)
+void DiffList::runDiff(const std::shared_ptr<LineDataVector>& p1, const size_t index1, LineRef size1, const std::shared_ptr<LineDataVector>& p2, const size_t index2, LineRef size2)
 {
     ProgressScope pp;
     static GnuDiff gnuDiff; // All values are initialized with zeros.
@@ -673,8 +650,8 @@ void DiffList::runDiff(const std::shared_ptr<LineDataVector> &p1, const size_t i
 
         gnuDiff.ignore_white_space = GnuDiff::IGNORE_ALL_SPACE; // I think nobody needs anything else ...
         gnuDiff.bIgnoreWhiteSpace = true;
-        gnuDiff.bIgnoreNumbers = pOptions->m_bIgnoreNumbers;
-        gnuDiff.minimal = pOptions->m_bTryHard;
+        gnuDiff.bIgnoreNumbers = gOptions->m_bIgnoreNumbers;
+        gnuDiff.minimal = gOptions->m_bTryHard;
         gnuDiff.ignore_case = false;
         GnuDiff::change* script = gnuDiff.diff_2_files(&comparisonInput);
 
@@ -684,11 +661,11 @@ void DiffList::runDiff(const std::shared_ptr<LineDataVector> &p1, const size_t i
         GnuDiff::change* p = nullptr;
         for(GnuDiff::change* e = script; e; e = p)
         {
-            Diff d((LineCount)(e->line0 - currentLine1), e->deleted, e->inserted);
+            Diff d((LineType)(e->line0 - currentLine1), e->deleted, e->inserted);
             assert(d.numberOfEquals() == e->line1 - currentLine2);
 
-            currentLine1 += (LineRef)(d.numberOfEquals() + d.diff1());
-            currentLine2 += (LineRef)(d.numberOfEquals() + d.diff2());
+            currentLine1 += LineRef((quint64)d.numberOfEquals() + d.diff1());
+            currentLine2 += LineRef((quint64)d.numberOfEquals() + d.diff2());
             assert(currentLine1 <= size1 && currentLine2 <= size2);
             push_back(d);
 
@@ -709,7 +686,7 @@ void DiffList::runDiff(const std::shared_ptr<LineDataVector> &p1, const size_t i
             currentLine1 += equalLinesAtStart;
             currentLine2 += equalLinesAtStart;
 
-            LineCount nofEquals = std::min(size1 - currentLine1, size2 - currentLine2);
+            LineType nofEquals = std::min(size1 - currentLine1, size2 - currentLine2);
             if(nofEquals == 0)
             {
                 back().adjustDiff1(size1 - currentLine1);
@@ -722,53 +699,48 @@ void DiffList::runDiff(const std::shared_ptr<LineDataVector> &p1, const size_t i
             }
         }
     }
-
+#ifndef NDEBUG
     verify(size1, size2);
-
+#endif
     ProgressProxy::setCurrent(1);
 }
 
+#ifndef NDEBUG
 // Verify difflist
 void DiffList::verify(const LineRef size1, const LineRef size2)
 {
-#ifndef NDEBUG
     LineRef l1 = 0;
     LineRef l2 = 0;
 
     for(const Diff& curDiff: *this)
     {
         assert(curDiff.numberOfEquals() >= 0);
-        assert(curDiff.diff1() <= TYPE_MAX(LineRef::LineType) && curDiff.diff2() <= TYPE_MAX(LineRef::LineType));
+        assert(curDiff.diff1() <= limits<LineType>::max() && curDiff.diff2() <= limits<LineType>::max());
         l1 += curDiff.numberOfEquals() + LineRef(curDiff.diff1());
         l2 += curDiff.numberOfEquals() + LineRef(curDiff.diff2());
     }
 
     assert(l1 == size1 && l2 == size2);
-#else
-    Q_UNUSED(size1); Q_UNUSED(size2);
-#endif
 }
+#endif
 
-void ManualDiffHelpList::runDiff(const std::shared_ptr<LineDataVector> &p1, LineRef size1, const std::shared_ptr<LineDataVector> &p2, LineRef size2, DiffList& diffList,
-                                 e_SrcSelector winIdx1, e_SrcSelector winIdx2,
-                                 const QSharedPointer<Options> &pOptions)
+void ManualDiffHelpList::runDiff(const std::shared_ptr<LineDataVector>& p1, LineRef size1, const std::shared_ptr<LineDataVector>& p2, LineRef size2, DiffList& diffList,
+                                 e_SrcSelector winIdx1, e_SrcSelector winIdx2)
 {
     diffList.clear();
     DiffList diffList2;
 
     qint32 l1begin = 0;
     qint32 l2begin = 0;
-    ManualDiffHelpList::const_iterator i;
-    for(i = begin(); i != end(); ++i)
-    {
-        const ManualDiffHelpEntry& mdhe = *i;
 
+    for(const ManualDiffHelpEntry& mdhe: *this)
+    {
         LineRef l1end = mdhe.getLine1(winIdx1);
         LineRef l2end = mdhe.getLine1(winIdx2);
 
         if(l1end.isValid() && l2end.isValid())
         {
-            diffList2.runDiff(p1, l1begin, l1end - l1begin, p2, l2begin, l2end - l2begin, pOptions);
+            diffList2.runDiff(p1, l1begin, l1end - l1begin, p2, l2begin, l2end - l2begin);
             diffList.splice(diffList.end(), diffList2);
             l1begin = l1end;
             l2begin = l2end;
@@ -780,14 +752,14 @@ void ManualDiffHelpList::runDiff(const std::shared_ptr<LineDataVector> &p1, Line
             {
                 ++l1end; // point to line after last selected line
                 ++l2end;
-                diffList2.runDiff(p1, l1begin, l1end - l1begin, p2, l2begin, l2end - l2begin, pOptions);
+                diffList2.runDiff(p1, l1begin, l1end - l1begin, p2, l2begin, l2end - l2begin);
                 diffList.splice(diffList.end(), diffList2);
                 l1begin = l1end;
                 l2begin = l2end;
             }
         }
     }
-    diffList2.runDiff(p1, l1begin, size1 - l1begin, p2, l2begin, size2 - l2begin, pOptions);
+    diffList2.runDiff(p1, l1begin, size1 - l1begin, p2, l2begin, size2 - l2begin);
     diffList.splice(diffList.end(), diffList2);
 }
 
@@ -1313,12 +1285,14 @@ void DiffList::calcDiff(const QString& line1, const QString& line2, const qint32
                     }
                 }
             }
-            //Bail this should never happen. Not a nice exit but acts as a back stop against harder to detect infine looping.
-            if(i1 == TYPE_MAX(qint32))
+            /*
+                Bail this should never happen.
+                Acts back stop against harder to detect overfollow issues.
+            */
+            if(Q_UNLIKELY(i1 == limits<decltype(i1)>::max()))
             {
                 assert(false);
-                abort();
-                return;
+                throw std::range_error("Too many diffs");
             }
         }
 
@@ -1432,6 +1406,30 @@ void DiffList::calcDiff(const QString& line1, const QString& line2, const qint32
     }
 #endif // !NDEBUG
 }
+//Compute fineDiff
+void DiffList::optimize()
+{
+    DiffList::iterator dli;
+    bool bUsefulFineDiff = false;
+    qint64 index = 0;
+
+    for(const Diff& diff: *this)
+    {
+        if(diff.numberOfEquals() >= 4)
+        {
+            bUsefulFineDiff = true;
+            break;
+        }
+    }
+
+    for(Diff& diff: *this)
+    {
+        if(!bUsefulFineDiff || index <= 0)
+            diff.refine();
+
+        ++index;
+    }
+}
 
 bool Diff3Line::fineDiff(bool inBTextsTotalEqual, const e_SrcSelector selector, const std::shared_ptr<LineDataVector> &v1, const std::shared_ptr<LineDataVector> &v2, const IgnoreFlags eIgnoreFlags)
 {
@@ -1460,10 +1458,13 @@ bool Diff3Line::fineDiff(bool inBTextsTotalEqual, const e_SrcSelector selector,
         k2 = getLineA();
     }
 
-    qDebug(kdiffCore) << "k1 = " << k1 << ", k2 = " << k2;
+    qCDebug(kdiffCore) << "k1 = " << k1 << ", k2 = " << k2;
     if((!k1.isValid() && k2.isValid()) || (k1.isValid() && !k2.isValid())) bTextsTotalEqual = false;
     if(k1.isValid() && k2.isValid())
     {
+        assert(((unsigned long)k1) <= (*v1).size() && (*v1)[k1].getBuffer() != nullptr);
+        assert(((unsigned long)k2) <= (*v2).size() && (*v2)[k2].getBuffer() != nullptr);
+
         if((*v1)[k1].size() != (*v2)[k2].size() || QString::compare((*v1)[k1].getLine(), (*v2)[k2].getLine()) != 0)
         {
             bTextsTotalEqual = false;
@@ -1471,27 +1472,7 @@ bool Diff3Line::fineDiff(bool inBTextsTotalEqual, const e_SrcSelector selector,
             pDiffList->calcDiff((*v1)[k1].getLine(), (*v2)[k2].getLine(), maxSearchLength);
 
             // Optimize the diff list.
-            DiffList::iterator dli;
-            bool bUsefulFineDiff = false;
-            for(const Diff& diff: *pDiffList)
-            {
-                if(diff.numberOfEquals() >= 4)
-                {
-                    bUsefulFineDiff = true;
-                    break;
-                }
-            }
-
-            for(dli = pDiffList->begin(); dli != pDiffList->end(); ++dli)
-            {
-                if(dli->numberOfEquals() < 4 && (dli->diff1() > 0 || dli->diff2() > 0) && !(bUsefulFineDiff && dli == pDiffList->begin()))
-                {
-                    dli->adjustDiff1(dli->numberOfEquals());
-                    dli->adjustDiff2(dli->numberOfEquals());
-                    dli->setNumberOfEquals(0);
-                }
-            }
-
+            pDiffList->optimize();
             setFineDiff(selector, pDiffList);
         }
         /*
@@ -1518,7 +1499,7 @@ bool Diff3Line::fineDiff(bool inBTextsTotalEqual, const e_SrcSelector selector,
 }
 
 void Diff3Line::getLineInfo(const e_SrcSelector winIdx, const bool isTriple, LineRef& lineIdx,
-                            std::shared_ptr<DiffList>& pFineDiff1, std::shared_ptr<DiffList>& pFineDiff2, // return values
+                            std::shared_ptr<const DiffList>& pFineDiff1, std::shared_ptr<const DiffList>& pFineDiff2, // return values
                             ChangeFlags& changed, ChangeFlags& changed2) const
 {
     changed = NoChange;
@@ -1588,10 +1569,10 @@ void Diff3LineList::calcDiff3LineVector(Diff3LineVector& d3lv)
 }
 
 // Just make sure that all input lines are in the output too, exactly once.
-void Diff3LineList::debugLineCheck(const LineCount size, const e_SrcSelector srcSelector) const
+void Diff3LineList::debugLineCheck(const LineType size, const e_SrcSelector srcSelector) const
 {
-    //FIXME:Does not work m_pOptions->m_bDiff3AlignBC is set.
-    LineCount i = 0;
+    //FIXME:Does not work when m_pOptions->m_bDiff3AlignBC is set.
+    LineType i = 0;
 
     for(const Diff3Line &entry: *this)
     {
@@ -1653,12 +1634,12 @@ void Diff3LineList::findHistoryRange(const QRegularExpression& historyStart, boo
     // Search for end of history
     for(range.end = range.start, range.endIdx = range.startIdx; range.end != end(); ++range.end, ++range.endIdx)
     {
-        QString sA = range.end->getString(e_SrcSelector::A);
-        QString sB = range.end->getString(e_SrcSelector::B);
-        QString sC = range.end->getString(e_SrcSelector::C);
-        if(!((sA.isEmpty() || historyLead == Utils::calcHistoryLead(sA)) &&
-             (sB.isEmpty() || historyLead == Utils::calcHistoryLead(sB)) &&
-             (!bThreeFiles || sC.isEmpty() || historyLead == Utils::calcHistoryLead(sC))))
+        const QString sA = range.end->getString(e_SrcSelector::A);
+        const QString sB = range.end->getString(e_SrcSelector::B);
+        const QString sC = range.end->getString(e_SrcSelector::C);
+        if((!sA.isEmpty() && historyLead != Utils::calcHistoryLead(sA)) ||
+           (!sB.isEmpty() && historyLead != Utils::calcHistoryLead(sB)) ||
+           (bThreeFiles && !sC.isEmpty() && historyLead != Utils::calcHistoryLead(sC)))
         {
             break; // End of the history
         }
