@@ -168,6 +168,7 @@ void ProgressDialog::endBackgroundTask()
         backgroundTaskCount--;
         if(backgroundTaskCount == 0)
         {
+            killJob();
             hide();
         }
     }
@@ -180,6 +181,7 @@ void ProgressDialog::pop(bool bRedrawUpdate)
         m_progressStack.pop_back();
         if(m_progressStack.empty())
         {
+            killJob();
             hide();
         }
         else
@@ -316,7 +318,9 @@ void ProgressDialog::enterEventLoop(KJob* pJob, const QString& jobInfo)
     if(m_eventLoop == nullptr)
     {
         m_eventLoop = QPointer<QEventLoop>(new QEventLoop(this));
-        m_eventLoop->exec(); // this function only returns after ProgressDialog::exitEventLoop() is called.
+        m_eventLoop->exec(); // this function returns after starting loop not what you might expect from the docs.
+        // Actually wait for loop to exit.
+        while(m_eventLoop->isRunning()) { m_eventLoop->processEvents(QEventLoop::WaitForMoreEvents); }
         m_eventLoop.clear();
     }
     else
@@ -408,13 +412,17 @@ void ProgressDialog::hide()
     m_delayedHideTimer = startTimer(100);
 }
 
-void ProgressDialog::delayedHide()
+void ProgressDialog::killJob()
 {
     if(m_pJob != nullptr)
     {
-        m_pJob->kill(KJob::Quietly);
+        m_pJob->kill(KJob::EmitResult);
         m_pJob = nullptr;
     }
+}
+
+void ProgressDialog::delayedHide()
+{
     QDialog::hide();
     dialogUi.information->setText("");
 
