@@ -1,3 +1,4 @@
+// clang-format off
 /*
  * KDiff3 - Text Diff And Merge Tool
  *
@@ -10,8 +11,8 @@
 
 #include "diff.h"
 #include "FileNameLineEdit.h"
-#include "options.h"
 #include "kdiff3.h"
+#include "options.h"
 #include "TypeUtils.h"
 #include "ui_opendialog.h"
 
@@ -36,16 +37,15 @@
 
 OpenDialog::OpenDialog(
     KDiff3App* pParent, const QString& n1, const QString& n2, const QString& n3,
-    bool bMerge, const QString& outputName,  const QSharedPointer<Options> &pOptions)
+    bool bMerge, const QString& outputName)
     : QDialog(pParent)
 {
     dialogUi.setupUi(this);
     setModal(true);
-    m_pOptions = pOptions;
     //Abort if verticalLayout is not the immediate child of the dialog. This interferes with re-sizing.
     assert(dialogUi.virticalLayout->parent() == this);
 
-    dialogUi.lineA->insertItems(0, m_pOptions->m_recentAFiles);
+    dialogUi.lineA->insertItems(0, gOptions->getRecentFilesA());
     dialogUi.lineA->setEditText(n1);
 
     QPushButton* button = dialogUi.fileSelectA;
@@ -55,7 +55,7 @@ OpenDialog::OpenDialog(
     chk_connect_a(dialogUi.lineA, &QComboBox::editTextChanged, this, &OpenDialog::inputFilenameChanged);
 
     dialogUi.lineB->setEditable(true);
-    dialogUi.lineB->insertItems(0, m_pOptions->m_recentBFiles);
+    dialogUi.lineB->insertItems(0, gOptions->getRecentFilesB());
     dialogUi.lineB->setEditText(n2);
 
     dialogUi.lineB->setMinimumWidth(200);
@@ -66,7 +66,7 @@ OpenDialog::OpenDialog(
     chk_connect_a(dialogUi.lineB, &QComboBox::editTextChanged, this, &OpenDialog::inputFilenameChanged);
 
     dialogUi.lineC->setEditable(true);
-    dialogUi.lineC->insertItems(0, m_pOptions->m_recentCFiles);
+    dialogUi.lineC->insertItems(0, gOptions->getRecentFilesC());
     dialogUi.lineC->setEditText(n3);
     dialogUi.lineC->setMinimumWidth(200);
     button = dialogUi.fileSelectC;
@@ -78,19 +78,19 @@ OpenDialog::OpenDialog(
     button = dialogUi.swapCopy;
 
     QMenu* m = new QMenu(this);
-    m->addAction(i18n("Swap %1<->%2", i18n("A"), i18n("B")));
-    m->addAction(i18n("Swap %1<->%2", i18n("B"), i18n("C")));
-    m->addAction(i18n("Swap %1<->%2", i18n("C"), i18n("A")));
-    m->addAction(i18n("Copy %1->Output", i18n("A")));
-    m->addAction(i18n("Copy %1->Output", i18n("B")));
-    m->addAction(i18n("Copy %1->Output", i18n("C")));
-    m->addAction(i18n("Swap %1<->Output", i18n("A")));
-    m->addAction(i18n("Swap %1<->Output", i18n("B")));
-    m->addAction(i18n("Swap %1<->Output", i18n("C")));
+    m->addAction(i18n("Swap %1<->%2", QStringLiteral("A"), QStringLiteral("B")));
+    m->addAction(i18n("Swap %1<->%2", QStringLiteral("B"), QStringLiteral("C")));
+    m->addAction(i18n("Swap %1<->%2", QStringLiteral("C"), QStringLiteral("A")));
+    m->addAction(i18n("Copy %1->Output", QStringLiteral("A")));
+    m->addAction(i18n("Copy %1->Output", QStringLiteral("B")));
+    m->addAction(i18n("Copy %1->Output", QStringLiteral("C")));
+    m->addAction(i18n("Swap %1<->Output", QStringLiteral("A")));
+    m->addAction(i18n("Swap %1<->Output", QStringLiteral("B")));
+    m->addAction(i18n("Swap %1<->Output", QStringLiteral("C")));
     chk_connect_a(m, &QMenu::triggered, this, &OpenDialog::slotSwapCopyNames);
     button->setMenu(m);
 
-    dialogUi.lineOut->insertItems(0, m_pOptions->m_recentOutputFiles);
+    dialogUi.lineOut->insertItems(0, gOptions->getRecentOutputFiles());
     dialogUi.lineOut->setEditText(outputName);
 
     button = dialogUi.selectOutputFile;
@@ -159,8 +159,8 @@ void OpenDialog::selectURL(QComboBox* pLine, bool bDir, qint32 i, bool bSave)
 
     currentUrl = QUrl::fromUserInput(current, QString(), QUrl::AssumeLocalFile);
     QUrl newURL = bDir ? QFileDialog::getExistingDirectoryUrl(this, i18n("Open Folder"), currentUrl)
-                       : bSave ? QFileDialog::getSaveFileUrl(this, i18n("Select Output File"), currentUrl, i18n("all/allfiles (*)"))
-                               : QFileDialog::getOpenFileUrl(this, i18n("Open File"), currentUrl, i18n("all/allfiles (*)"));
+                       : bSave ? QFileDialog::getSaveFileUrl(this, i18n("Select Output File"), currentUrl)
+                               : QFileDialog::getOpenFileUrl(this, i18n("Open File"), currentUrl);
     if(!newURL.isEmpty())
     {
         /*
@@ -202,7 +202,7 @@ void OpenDialog::inputFilenameChanged()
 void OpenDialog::fixCurrentText(QComboBox* pCB)
 {
     QString s = pCB->currentText();
-    qint32 pos = s.indexOf('\n');
+    QtSizeType pos = s.indexOf('\n');
 
     if(pos >= 0)
         s = s.left(pos);
@@ -215,85 +215,70 @@ void OpenDialog::fixCurrentText(QComboBox* pCB)
 
 void OpenDialog::accept()
 {
-    qint32 maxNofRecentFiles = 10;
     fixCurrentText(dialogUi.lineA);
-
     QString s = dialogUi.lineA->currentText();
-    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QString(), QUrl::AssumeLocalFile));
-    QStringList* sl = &m_pOptions->m_recentAFiles;
-    // If an item exist, remove it from the list and reinsert it at the beginning.
-    sl->removeAll(s);
-    if(!s.isEmpty()) sl->prepend(s);
-    if(sl->count() > maxNofRecentFiles) sl->erase(sl->begin() + maxNofRecentFiles, sl->end());
+    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QDir::currentPath(), QUrl::AssumeLocalFile));
+    gOptions->getRecentFilesA().push_front(s);
 
     fixCurrentText(dialogUi.lineB);
     s = dialogUi.lineB->currentText();
-    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QString(), QUrl::AssumeLocalFile));
-    sl = &m_pOptions->m_recentBFiles;
-    sl->removeAll(s);
-    if(!s.isEmpty()) sl->prepend(s);
-    if(sl->count() > maxNofRecentFiles) sl->erase(sl->begin() + maxNofRecentFiles, sl->end());
+    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QDir::currentPath(), QUrl::AssumeLocalFile));
+    gOptions->getRecentFilesB().push_front(s);
 
     fixCurrentText(dialogUi.lineC);
     s = dialogUi.lineC->currentText();
-    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QString(), QUrl::AssumeLocalFile));
-    sl = &m_pOptions->m_recentCFiles;
-    sl->removeAll(s);
-    if(!s.isEmpty()) sl->prepend(s);
-    if(sl->count() > maxNofRecentFiles) sl->erase(sl->begin() + maxNofRecentFiles, sl->end());
+    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QDir::currentPath(), QUrl::AssumeLocalFile));
+    gOptions->getRecentFilesC().push_front(s);
 
     fixCurrentText(dialogUi.lineOut);
     s = dialogUi.lineOut->currentText();
-    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QString(), QUrl::AssumeLocalFile));
-    sl = &m_pOptions->m_recentOutputFiles;
-    sl->removeAll(s);
-    if(!s.isEmpty()) sl->prepend(s);
-    if(sl->count() > maxNofRecentFiles) sl->erase(sl->begin() + maxNofRecentFiles, sl->end());
+    s = FileAccess::prettyAbsPath(QUrl::fromUserInput(s, QDir::currentPath(), QUrl::AssumeLocalFile));
+    gOptions->getRecentOutputFiles().push_front(s);
 
     QDialog::accept();
 }
 
 void OpenDialog::slotSwapCopyNames(QAction* pAction) const // id selected in the popup menu
 {
-    qint32 id = pAction->parentWidget()->actions().indexOf(pAction);
+    const QtSizeType id = pAction->parentWidget()->actions().indexOf(pAction);
     QComboBox* cb1 = nullptr;
     QComboBox* cb2 = nullptr;
 
     switch(id)
     {
-        case 0:
+        case swapAB:
             cb1 = dialogUi.lineA;
             cb2 = dialogUi.lineB;
             break;
-        case 1:
+        case swapBC:
             cb1 = dialogUi.lineB;
             cb2 = dialogUi.lineC;
             break;
-        case 2:
+        case swapCA:
             cb1 = dialogUi.lineC;
             cb2 = dialogUi.lineA;
             break;
-        case 3:
+        case copyAToOut:
             cb1 = dialogUi.lineA;
             cb2 = dialogUi.lineOut;
             break;
-        case 4:
+        case copyBToOut:
             cb1 = dialogUi.lineB;
             cb2 = dialogUi.lineOut;
             break;
-        case 5:
+        case copyCToOut:
             cb1 = dialogUi.lineC;
             cb2 = dialogUi.lineOut;
             break;
-        case 6:
+        case swapAOut:
             cb1 = dialogUi.lineA;
             cb2 = dialogUi.lineOut;
             break;
-        case 7:
+        case swapBOut:
             cb1 = dialogUi.lineB;
             cb2 = dialogUi.lineOut;
             break;
-        case 8:
+        case swapCOut:
             cb1 = dialogUi.lineC;
             cb2 = dialogUi.lineOut;
             break;
@@ -304,7 +289,7 @@ void OpenDialog::slotSwapCopyNames(QAction* pAction) const // id selected in the
         QString t1 = cb1->currentText();
         QString t2 = cb2->currentText();
         cb2->setEditText(t1);
-        if(id <= 2 || id >= 6)
+        if(id <= swapCA || id >= swapAOut)
         {
             cb1->setEditText(t2);
         }
@@ -354,8 +339,8 @@ FindDialog::FindDialog(QWidget* pParent)
     QDialogButtonBox *box = new QDialogButtonBox(QDialogButtonBox::Cancel, this);
     layout->addWidget(box, line, 0, 1, 2);
     box->addButton(i18n("&Search"), QDialogButtonBox::AcceptRole);
-    chk_connect(box, &QDialogButtonBox::accepted, this, &FindDialog::accept);
-    chk_connect(box, &QDialogButtonBox::rejected, this, &FindDialog::reject);
+    chk_connect_a(box, &QDialogButtonBox::accepted, this, &FindDialog::accept);
+    chk_connect_a(box, &QDialogButtonBox::rejected, this, &FindDialog::reject);
 
     hide();
 }
@@ -389,15 +374,15 @@ RegExpTester::RegExpTester(QWidget* pParent, const QString& autoMergeRegExpToolT
     l->setToolTip(autoMergeRegExpToolTip);
     m_pAutoMergeRegExpEdit = new QLineEdit(this);
     pGrid->addWidget(m_pAutoMergeRegExpEdit, line, 1);
-    chk_connect(m_pAutoMergeRegExpEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
+    chk_connect_a(m_pAutoMergeRegExpEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
     ++line;
 
     l = new QLabel(i18n("Example auto merge line:"), this);
     pGrid->addWidget(l, line, 0);
-    l->setToolTip(i18n("To test auto merge, copy a line as used in your files."));
+    l->setToolTip(i18nc("Tool Tip", "To test auto merge, copy a line as used in your files."));
     m_pAutoMergeExampleEdit = new QLineEdit(this);
     pGrid->addWidget(m_pAutoMergeExampleEdit, line, 1);
-    chk_connect(m_pAutoMergeExampleEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
+    chk_connect_a(m_pAutoMergeExampleEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
     ++line;
 
     l = new QLabel(i18n("Match result:"), this);
@@ -416,16 +401,16 @@ RegExpTester::RegExpTester(QWidget* pParent, const QString& autoMergeRegExpToolT
     l->setToolTip(historyStartRegExpToolTip);
     m_pHistoryStartRegExpEdit = new QLineEdit(this);
     pGrid->addWidget(m_pHistoryStartRegExpEdit, line, 1);
-    chk_connect(m_pHistoryStartRegExpEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
+    chk_connect_a(m_pHistoryStartRegExpEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
     ++line;
 
     l = new QLabel(i18n("Example history start line (with leading comment):"), this);
     pGrid->addWidget(l, line, 0);
-    l->setToolTip(i18n("Copy a history start line as used in your files,\n"
-                       "including the leading comment."));
+    l->setToolTip(i18nc("Tool Tip", "Copy a history start line as used in your files,\n"
+                                    "including the leading comment."));
     m_pHistoryStartExampleEdit = new QLineEdit(this);
     pGrid->addWidget(m_pHistoryStartExampleEdit, line, 1);
-    chk_connect(m_pHistoryStartExampleEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
+    chk_connect_a(m_pHistoryStartExampleEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
     ++line;
 
     l = new QLabel(i18n("Match result:"), this);
@@ -444,7 +429,7 @@ RegExpTester::RegExpTester(QWidget* pParent, const QString& autoMergeRegExpToolT
     l->setToolTip(historyEntryStartRegExpToolTip);
     m_pHistoryEntryStartRegExpEdit = new QLineEdit(this);
     pGrid->addWidget(m_pHistoryEntryStartRegExpEdit, line, 1);
-    chk_connect(m_pHistoryEntryStartRegExpEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
+    chk_connect_a(m_pHistoryEntryStartRegExpEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
     ++line;
 
     l = new QLabel(i18n("History sort key order:"), this);
@@ -452,16 +437,16 @@ RegExpTester::RegExpTester(QWidget* pParent, const QString& autoMergeRegExpToolT
     l->setToolTip(historySortKeyOrderToolTip);
     m_pHistorySortKeyOrderEdit = new QLineEdit(this);
     pGrid->addWidget(m_pHistorySortKeyOrderEdit, line, 1);
-    chk_connect(m_pHistorySortKeyOrderEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
+    chk_connect_a(m_pHistorySortKeyOrderEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
     ++line;
 
     l = new QLabel(i18n("Example history entry start line (without leading comment):"), this);
     pGrid->addWidget(l, line, 0);
-    l->setToolTip(i18n("Copy a history entry start line as used in your files,\n"
-                       "but omit the leading comment."));
+    l->setToolTip(i18nc("Tool Tip", "Copy a history entry start line as used in your files,\n"
+                                    "but omit the leading comment."));
     m_pHistoryEntryStartExampleEdit = new QLineEdit(this);
     pGrid->addWidget(m_pHistoryEntryStartExampleEdit, line, 1);
-    chk_connect(m_pHistoryEntryStartExampleEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
+    chk_connect_a(m_pHistoryEntryStartExampleEdit, &QLineEdit::textChanged, this, &RegExpTester::slotRecalc);
     ++line;
 
     l = new QLabel(i18n("Match result:"), this);
@@ -480,8 +465,8 @@ RegExpTester::RegExpTester(QWidget* pParent, const QString& autoMergeRegExpToolT
 
     QDialogButtonBox *box = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, this);
     pGrid->addWidget(box, line, 0, 1, 2);
-    chk_connect(box, &QDialogButtonBox::accepted, this, &RegExpTester::accept);
-    chk_connect(box, &QDialogButtonBox::rejected, this, &RegExpTester::reject);
+    chk_connect_a(box, &QDialogButtonBox::accepted, this, &RegExpTester::accept);
+    chk_connect_a(box, &QDialogButtonBox::rejected, this, &RegExpTester::reject);
 
     resize(800, sizeHint().height());
 }
@@ -494,29 +479,29 @@ void RegExpTester::init(const QString& autoMergeRegExp, const QString& historySt
     m_pHistorySortKeyOrderEdit->setText(historySortKeyOrder);
 }
 
-QString RegExpTester::autoMergeRegExp()
+QString RegExpTester::autoMergeRegExp() const
 {
     return m_pAutoMergeRegExpEdit->text();
 }
 
-QString RegExpTester::historyStartRegExp()
+QString RegExpTester::historyStartRegExp() const
 {
     return m_pHistoryStartRegExpEdit->text();
 }
 
-QString RegExpTester::historyEntryStartRegExp()
+QString RegExpTester::historyEntryStartRegExp() const
 {
     return m_pHistoryEntryStartRegExpEdit->text();
 }
 
-QString RegExpTester::historySortKeyOrder()
+QString RegExpTester::historySortKeyOrder() const
 {
     return m_pHistorySortKeyOrderEdit->text();
 }
 
 void RegExpTester::slotRecalc()
 {
-    QRegularExpression autoMergeRegExp(m_pAutoMergeRegExpEdit->text());
+    const QRegularExpression autoMergeRegExp(m_pAutoMergeRegExpEdit->text());
     QRegularExpressionMatch match = autoMergeRegExp.match(m_pAutoMergeExampleEdit->text());
 
     if(match.hasMatch())
@@ -528,7 +513,7 @@ void RegExpTester::slotRecalc()
         m_pAutoMergeMatchResult->setText(i18n("Match failed."));
     }
 
-    QRegularExpression historyStartRegExp(m_pHistoryStartRegExpEdit->text());
+    const QRegularExpression historyStartRegExp(m_pHistoryStartRegExpEdit->text());
     match = historyStartRegExp.match(m_pHistoryStartExampleEdit->text());
 
     if(match.hasMatch())
@@ -548,8 +533,8 @@ void RegExpTester::slotRecalc()
         m_pHistorySortKeyResult->setText("");
         return;
     }
-    QRegularExpression historyEntryStartRegExp(m_pHistoryEntryStartRegExpEdit->text());
-    QString s = m_pHistoryEntryStartExampleEdit->text();
+    const QRegularExpression historyEntryStartRegExp(m_pHistoryEntryStartRegExpEdit->text());
+    const QString s = m_pHistoryEntryStartExampleEdit->text();
     match = historyEntryStartRegExp.match(s);
 
     if(match.hasMatch())
